﻿using Bogus;
using Infragistics.Windows.DataPresenter;
using PropertyChanged;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace Infragistics.Tests
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            this.DataContext = new Context();
        }

        private void XamDataGrid_FieldLayoutInitialized(object sender, Windows.DataPresenter.Events.FieldLayoutInitializedEventArgs e)
        {
            for (int i = 0; i < 140; i++)
            {
                e.FieldLayout.Fields.Add(new Field { Name = "Day " + (i + 1), Label = "Day " + (i + 1), AlternateBinding = new Binding { Path = new PropertyPath("Data[" + i + "]") } });
            }
        }
    }

    public class Context : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        public ObservableCollection<Item> Items
        {
            get;
            set;
        }

        public Context()
        {
            this.Items = new ObservableCollection<Item>(new Faker<Item>().RuleFor(p => p.Data, f => f.Lorem.Words(140).Select(t => new SubItem(t)).ToArray()).Generate(1000));
        }
    }

    public class SubItem : INotifyPropertyChanged
    {
        private string text;
        private bool isSelected;

        public event PropertyChangedEventHandler PropertyChanged;

        public SubItem(string text)
        {
            this.text = text;
        }

        public string Text => text;
        public bool IsSelected { get => isSelected; set => isSelected = value; }
    }

    public class Item : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        public SubItem[] Data { get; set; }
    }

    public class CustomControl : FrameworkElement
    {
        static int instanceCount;
        int instance;
        int changeCount;

        public CustomControl()
        {
            instance = ++instanceCount;
            this.DataContextChanged += this.CustomControl_DataContextChanged;
            this.ClipToBounds = true;
            this.MinHeight = 25;
        }

        private void CustomControl_DataContextChanged(object sender, DependencyPropertyChangedEventArgs e)
        {
            changeCount++;
            if (e.OldValue is INotifyPropertyChanged old)
            {
                old.PropertyChanged -= Data_PropertyChanged;
                this.cachedText = null;
                this.isSelected = false;
            }

            if (e.NewValue is SubItem data)
            {
                data.PropertyChanged += Data_PropertyChanged;
                this.cachedText = new FormattedText(data.Text, CultureInfo.CurrentCulture, FlowDirection.LeftToRight, new Typeface("Segoe UI"), 12, Brushes.Black, VisualTreeHelper.GetDpi(this).PixelsPerDip);
                this.isSelected = data.IsSelected;
                InvalidateVisual();
            }
            if (e.OldValue == null && e.NewValue != null)
            {
                Debug.WriteLine($"(re)init {changeCount} {instance}/{instanceCount}");
            }
            else if (e.OldValue != null && e.NewValue != null)
            {
                Debug.WriteLine($"  change {changeCount} {instance}/{instanceCount}");
            }
            else if (e.NewValue == null && e.OldValue != null)
            {
                Debug.WriteLine($" discard {changeCount} {instance}/{instanceCount}");
            }
        }

        private void Data_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            this.isSelected = ((SubItem)sender).IsSelected;
            this.cachedText = new FormattedText(((SubItem)sender).Text, CultureInfo.CurrentCulture, FlowDirection.LeftToRight, new Typeface("Segoe UI"), 12, Brushes.Black, VisualTreeHelper.GetDpi(this).PixelsPerDip);
            InvalidateVisual();
        }

        FormattedText cachedText;
        bool isSelected;

        protected override void OnRender(DrawingContext drawingContext)
        {
            if (this.isSelected)
                drawingContext.DrawRectangle(Brushes.LightBlue, null, new Rect(RenderSize));
            if (this.cachedText != null)
                drawingContext.DrawText(this.cachedText, new Point(0, (this.RenderSize.Height - this.cachedText.Height) / 2));
        }
    }
}
